#include "CivInpDbHelper.h"
#include "CivTableFields.h"
#include "CivConnection.h"
#include "CivPgDbConnection.h"

#include <algorithm>

CivInpDbHelper::CivInpDbHelper(const std::string& uri)
	:CivInpHelperAbs(uri)
{
	mCondtion = "1=1 ";
	isDelayTime = true;
}

CivInpDbHelper::~CivInpDbHelper()
{
}

bool CivInpDbHelper::getNode(CivNode& node)
{
	JunctionTable nodeTableFields;
	std::vector<std::string> fields;

	fields.push_back(nodeTableFields.sn);
	fields.push_back(nodeTableFields.elev);
	fields.push_back(nodeTableFields.baseDemand);
	fields.push_back(nodeTableFields.demandPattern);

	std::vector<std::map<std::string,std::string>> resultVector;	
	mDbConn->query(PIPENODE, fields, resultVector, mCondtion);

	// ༭װ
	size_t total = resultVector.size();
	for (int i = 0; i < total; i++)
	{
		std::map<std::string, std::string> map = resultVector[i];
		CivNode::NodeTable nodeTable;

		nodeTable.ID = map.find(nodeTableFields.sn)->second;
		nodeTable.Elev = map.find(nodeTableFields.elev)->second;

		std::string patternID = map.find(nodeTableFields.demandPattern)->second;
		std::string demand = map.find(nodeTableFields.baseDemand)->second;

		// ʱģ
		handleDelayPattern(demand, patternID, nodeTable.Demand, nodeTable.PatternId);
		
		node.addItem(nodeTable);
	}

	// ӴķŲĽڵ
	size_t size = mNodeVec.size();
	for (int i = 0; i < size; i++)
	{
		node.addItem(mNodeVec[i]);
	}
	return true;
}

bool CivInpDbHelper::getPipe(CivPipe& pipes)
{
	PipeTable pipeTable;

	std::vector<std::string> fields;
	fields.push_back(pipeTable.snNo); 
	fields.push_back(pipeTable.startPoint);
	fields.push_back(pipeTable.endPoint); 
	fields.push_back(pipeTable.length); 
	fields.push_back(pipeTable.diameter); 
	fields.push_back(pipeTable.friction); 
	fields.push_back(pipeTable.localLoss);
	fields.push_back(pipeTable.status);
	fields.push_back(pipeTable.code);
	
	std::vector<std::map<std::string, std::string>> resultVector;

	std::string condition = "\"from_layer\" not in ('','ˮ' ) and \"to_layer\" not in ('','ˮ' )";
	mDbConn->query(PIPELINE, fields, resultVector, condition);

	size_t total = resultVector.size();
    for (int i = 0; i < total; i++)
    {
        CivPipe::PipesTable pipe;

		std::map<std::string, std::string> map = resultVector[i];

		pipe.ID = map.find(pipeTable.snNo)->second;
		pipe.Node1 = map.find(pipeTable.startPoint)->second;
		pipe.Node2 = map.find(pipeTable.endPoint)->second;
		pipe.Length = map.find(pipeTable.length)->second;
		pipe.Diameter = map.find(pipeTable.diameter)->second;
		pipe.Roughness = map.find(pipeTable.friction)->second;
		pipe.MinorLoss = map.find(pipeTable.localLoss)->second;
		pipe.Status = map.find(pipeTable.status)->second;
		pipe.code = map.find(pipeTable.code)->second;
		pipes.addItem(pipe);
    }
	// Ĺܶ
	size_t size = mPipesVec.size();
	for (int i = 0; i < size; i++)
	{
		pipes.addItem(mPipesVec[i]);
	}
	return true;
}

bool CivInpDbHelper::getTank(CivTank& tanks)
{
	TankTable tankTable;

	std::vector<std::string> fields;

	fields.push_back(tankTable.sn);
	fields.push_back(tankTable.elev);
	fields.push_back(tankTable.initLevel);
	fields.push_back(tankTable.minLevel);
	fields.push_back(tankTable.maxLevel);
	fields.push_back(tankTable.diametor);
	fields.push_back(tankTable.minVolume);
	fields.push_back(tankTable.volumeCurve);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(TANK, fields, resultVector, mCondtion);

	size_t total = resultVector.size();
	for (int i = 0; i < total; i++)
	{
		CivTank::TankTable tank;
		std::map<std::string, std::string>map = resultVector[i];

		tank.ID = map.find(tankTable.sn)->second;
		tank.Elev = map.find(tankTable.elev)->second;
		tank.InitLevel = map.find(tankTable.initLevel)->second;
		tank.MinLevel = map.find(tankTable.minLevel)->second;
		tank.MaxLevel = map.find(tankTable.maxLevel)->second;
		tank.Diameter = map.find(tankTable.diametor)->second;
		tank.MinVol = map.find(tankTable.minVolume)->second;
		tank.VolCurve = map.find(tankTable.volumeCurve)->second;
		
		tanks.addItem(tank);
	}
	return true;
}

bool CivInpDbHelper::getValve(CivValve& valves)
{
	size_t size = mValuvesVec.size();
	for (int i = 0; i < size; i++)
	{
		valves.addItem(mValuvesVec[i]);
	}
	return true;
}

bool CivInpDbHelper::getPumps(CivPumps& pumps)
{
	PumpTable pmTable;
	std::vector<std::string> fields;

	fields.push_back(pmTable.sn);
	fields.push_back(pmTable.startPoint);
	fields.push_back(pmTable.endPoint);
	fields.push_back(pmTable.headCurve);
	fields.push_back(pmTable.ratio);
	fields.push_back(pmTable.power);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(PUMP, fields, resultVector, mCondtion);
	
	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivPumps::PumpTable pump;
		std::map<std::string, std::string> map = resultVector[i];

		pump.ID = map.find(pmTable.sn)->second;
		pump.Node1 = map.find(pmTable.startPoint)->second;
		pump.Node2 = map.find(pmTable.endPoint)->second;
		std::string param;
		
		std::string curve = map.find(pmTable.headCurve)->second;
		if (curve != "")
			param.append("head " + curve);

		std::string power = map.find(pmTable.power)->second;
		if (power != "")
			param.append(" power " + power);
	
		std::string speedRatio = map.find(pmTable.ratio)->second;
		if (speedRatio != "")
			param.append(" SPEED " + speedRatio);

		pump.Parameters = param;
		pumps.addItem(pump);
	}
	return true;
}

bool CivInpDbHelper::getReservoirs(CivReservoirs& reservoirs)
{
	ResourcesTable restemTable;

	std::vector<std::string> fields;

	fields.push_back(restemTable.sn);
	fields.push_back(restemTable.totalHead);
	fields.push_back(restemTable.headPattern);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(RESIVOIR, fields, resultVector, mCondtion);

	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivReservoirs::ReservoirsTable resTable;
		std::map<std::string, std::string> map = resultVector[i];

		resTable.ID = map.find(restemTable.sn)->second;
		resTable.Head = map.find(restemTable.totalHead)->second;
		resTable.Pattern = map.find(restemTable.headPattern)->second;

		reservoirs.addItem(resTable);
	}
	return true;
}

bool CivInpDbHelper::getCoordinates(CivCoordinates& coord)
{
	JunctionTable nodeTable;

	std::vector<std::string> fields;

	fields.push_back(nodeTable.sn);
	fields.push_back(nodeTable.xCoord);
	fields.push_back(nodeTable.yCoord);
	fields.push_back(nodeTable.code);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(PIPENODE, fields, resultVector, mCondtion);
	mDbConn->query(TANK, fields, resultVector, mCondtion);
	mDbConn->query(RESIVOIR, fields, resultVector, mCondtion);

	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivCoordinates::CoordTable coordTable;
		std::map<std::string, std::string> map = resultVector[i];

		coordTable.ID = map.find(fields[0])->second;
		coordTable.XCoord = map.find(fields[1])->second;
		coordTable.YCoord = map.find(fields[2])->second;
		coordTable.code = map.find(nodeTable.code)->second;

		coord.addItem(coordTable);
	}
	// 뷧ŴĽڵ
	size_t size = mCoords.size();
	for (int i = 0; i < size; i++)
	{
		coord.addItem(mCoords[i]);
	}

	return true;
}

bool CivInpDbHelper::getQuality(CivQuality& quality)
{
	JunctionTable nodeTable;

	std::vector<std::string> fields;

	fields.push_back(nodeTable.sn);
	fields.push_back(nodeTable.initQuality);
	fields.push_back(nodeTable.chlorine);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(PIPENODE, fields, resultVector, mCondtion);
	mDbConn->query(RESIVOIR, fields, resultVector,mCondtion);
	mDbConn->query(TANK, fields, resultVector, mCondtion);

	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivQuality::QualityTable quliTable;

		std::map<std::string, std::string> map = resultVector[i];
		quliTable.ID = map.find(nodeTable.sn)->second;
		switch (mQualityType)
		{
		
		case Chlorine:
			quliTable.InitQuality = map.find(nodeTable.chlorine)->second;
			break;
		case Age:
		default:
			quliTable.InitQuality = map.find(nodeTable.initQuality)->second;
			break;
		}

		quality.addItem(quliTable);
	}

	return true;
}

bool CivInpDbHelper::getStatus(CivStatus& status)
{
	// ״ֵ̬
	size_t statuSize = mStatusVec.size();
	for (int i = 0; i < statuSize; i++)
	{
		status.addItem(mStatusVec[i]);
	}
	
	return true;
}

void CivInpDbHelper::handlePump()
{
	
}

bool CivInpDbHelper::handleValves2()
{
	std::string sql = "SELECT A.*,B.* FROM\
		(\
			SELECT\
			node.\"\",\
			node.\"\" AS ź,\
			node.\"\" AS ,\
			node.\"̶״̬\" AS Ź̶״̬,\
			node.\"\" AS ,\
			node.\"\" AS ,\
			node.\"ˮϵ\" AS ˮϵ,\
			pipe. AS ιܶα,\
			pipe.\"ܳ\" AS ιܳ,\
			pipe.\"ܾ\" AS ιܾ,\
			pipe.\"Ħϵ\" AS ιܶĦϵ,\
			junc.\"\" AS νڵ,\
			junc.\"code\" AS νڵcode,\
			junc.\"߳\" AS νڵ߳,\
			junc.\"ˮģʽ\" AS νڵˮģʽ,\
			junc.\"ˮ\" as νڵˮ,\
			junc.\"\" AS νڵ,\
			junc.\"\" AS νڵ,\
			junc.\"ʼˮ\" AS νڵʼˮ\
			FROM\
			 AS node, ܶ AS pipe, ڵ as junc\
			WHERE\
			pipe.\"from_code\" = node.\"code\"  AND pipe.\"to_code\" = junc.\"code\"\
			) A,\
		(\
			SELECT\
			pipe. AS ιܶα,\
			pipe.\"ܳ\" AS ιܶιܳ,\
			pipe.\"ܾ\" AS ιܶιܾ,\
			pipe.\"Ħϵ\" AS ιܶĦϵ,\
			node.\"\",\
			junc.\"\" AS νڵ,\
			junc.\"code\" AS νڵcode,\
			junc.\"߳\" AS νڵ߳,\
			junc.\"ˮģʽ\" AS νڵˮģʽ,\
			junc.\"ˮ\" as νڵˮ,\
			junc.\"\" AS νڵ,\
			junc.\"\" AS νڵ,\
			junc.\"ʼˮ\" AS νڵʼˮ\
			FROM\
			 AS node, ܶ AS pipe, ڵ as junc\
			WHERE\
			pipe.\"to_code\" = node.\"code\" AND pipe.\"from_code\" = junc.\"code\"\
			) B\
		WHERE\
		A.\"\" = b.\"\"";

	if (!mDbConn->execSql(sql))
	{
		std::string error = mDbConn->getLastError();
		return false;
	}

	// ѯ뷧ŹĹܶΣͽڵ
	std::vector<std::map<std::string, std::string>> valvesPipes;
	mDbConn->queryResult(valvesPipes);

	// ܸ
	size_t total = valvesPipes.size();
	for (int i = 0; i < total; i++)
	{
		std::map<std::string,std::string> valvePipe = valvesPipes[i];

		std::string vaSzNo = valvePipe.find("")->second;
		std::string vaXcoord = valvePipe.find("ź")->second;
		std::string vaYcoord = valvePipe.find("")->second;
		std::string vaStatus = valvePipe.find("Ź̶״̬")->second;
		std::string vaType = valvePipe.find("")->second;
		std::string vaSetting = valvePipe.find("")->second;
		std::string vaLoss = valvePipe.find("ˮϵ")->second;
		std::string upperPipeSzNo = valvePipe.find("ιܶα")->second;
		std::string upperPipeCode = valvePipe.find("ιܶcode")->second;
		std::string upperPipeLength = valvePipe.find("ιܳ")->second;
		std::string upperPipeDiametor = valvePipe.find("ιܾ")->second;
		std::string upperPipeFriction = valvePipe.find("ιܶĦϵ")->second;
		std::string upperNodeSz = valvePipe.find("νڵ")->second;
		std::string upperNodeHeight = valvePipe.find("νڵ߳")->second;
		std::string upperNodePattern = valvePipe.find("νڵˮģʽ")->second;
		std::string upperNodeDemand = valvePipe.find("νڵˮ")->second;
		std::string upperNodeXcoord = valvePipe.find("νڵ")->second;
		std::string upperNodeYcoord = valvePipe.find("νڵ")->second;
		std::string upperNodeQuality = valvePipe.find("νڵʼˮ")->second;
		std::string donwPipeSz = valvePipe.find("ιܶα")->second;
		std::string donwPipeCode = valvePipe.find("ιܶcode")->second;
		std::string donwPipeLength = valvePipe.find("ιܶιܳ")->second;
		std::string donwPipeDiametor = valvePipe.find("ιܶιܾ")->second;
		std::string downPipeFraction = valvePipe.find("ιܶĦϵ")->second;
		std::string donwNodeSz = valvePipe.find("νڵ")->second;
		std::string donwNodeHeight = valvePipe.find("νڵ߳")->second;
		std::string donwNodePattern = valvePipe.find("νڵˮģʽ")->second;
		std::string donwNodeDemand = valvePipe.find("νڵˮ")->second;
		std::string donwNodeXcoord = valvePipe.find("νڵ")->second;
		std::string donwNodeYcoord = valvePipe.find("νڵ")->second;
		std::string donwNodeQuality = valvePipe.find("νڵʼˮ")->second;

		// жϷ
		std::string upperVaType = vaType;
		std::transform(vaType.begin(), vaType.begin(), vaType.end(), ::toupper);

		// Ҫڵ㣬ڵ㾡ܽӽ
		CivNode::NodeTable startNodeTable;
		CivCoordinates::CoordTable startNodeCoord; // ڵ

		// ڵϢ
		startNodeTable.ID = "VUP"+vaSzNo;
		startNodeTable.Elev = upperNodeHeight;
		startNodeTable.Demand = "0";

		// ֱ֮
		double vupperXcoord = (atof(upperNodeXcoord.c_str()) - atof(vaXcoord.c_str()))/5+atof(vaXcoord.c_str());
		double vupperYcoord = (atof(upperNodeYcoord.c_str()) - atof(vaYcoord.c_str()))/5 + atof(vaYcoord.c_str());

		// ڵ
		startNodeCoord.ID = "VUP" + vaSzNo;
		startNodeCoord.XCoord = std::to_string(vupperXcoord);
		startNodeCoord.YCoord = std::to_string(vupperYcoord);
		
		// ֹڵ㹹
		CivNode::NodeTable endNodeTable; // ֹڵ
		CivCoordinates::CoordTable endNodeCoord; // ֹڵ

		// ֹڵϢ
		endNodeTable.ID = "VDW" + vaSzNo;
		endNodeTable.Elev = upperNodeHeight;
		endNodeTable.Demand = "0";

		mNodeVec.push_back(startNodeTable);
		mCoords.push_back(startNodeCoord);

		// ֱ֮
		double vdownXcoord = (atof(donwNodeXcoord.c_str()) - atof(vaXcoord.c_str())) / 5 + atof(vaXcoord.c_str());
		double vdownYcoord = (atof(donwNodeYcoord.c_str()) - atof(vaYcoord.c_str())) / 5 + atof(vaYcoord.c_str());

		// ֹڵϢ
		endNodeCoord.ID = "VDW" + vaSzNo;
		endNodeCoord.XCoord = std::to_string(vdownXcoord);
		endNodeCoord.YCoord = std::to_string(vdownYcoord);

		mNodeVec.push_back(endNodeTable);
		mCoords.push_back(endNodeCoord);

		// ιܶε
		CivPipe::PipesTable upperPipeTable;
		CivPipe::PipesTable downPipeTable;

		// ιܶ
		upperPipeTable.code = upperPipeCode;
		upperPipeTable.ID = upperPipeSzNo;
		upperPipeTable.Node1 = upperNodeSz;
		upperPipeTable.Node2 = startNodeTable.ID;
		upperPipeTable.Length = upperPipeLength;
		upperPipeTable.Diameter = upperPipeDiametor;
		upperPipeTable.Roughness = upperPipeFriction;
		upperPipeTable.Status = "Open";
		upperPipeTable.MinorLoss = vaLoss;

		// ιܶ
		downPipeTable.code = donwPipeCode;
		downPipeTable.ID = donwPipeSz;
		downPipeTable.Node1 =  endNodeTable.ID;
		downPipeTable.Node2 = donwNodeSz;
		downPipeTable.Length = donwPipeLength;
		downPipeTable.Diameter = donwPipeDiametor;
		downPipeTable.Roughness = upperPipeFriction;
		downPipeTable.Status = "Open";
		downPipeTable.MinorLoss = vaLoss;

		mPipesVec.push_back(downPipeTable);
		mPipesVec.push_back(upperPipeTable);
		
		// ͨŵЧڹܶ
		if (upperVaType == "BV")
		{
			// һܶ
			CivPipe::PipesTable vPipeTable;
			vPipeTable.ID = "VG"+vaSzNo;
			vPipeTable.Node1 = startNodeTable.ID;
			vPipeTable.Node2 = endNodeTable.ID;
			vPipeTable.Length = donwPipeLength;
			vPipeTable.Diameter = donwPipeDiametor;
			vPipeTable.Roughness = upperPipeFriction;
			vPipeTable.Status = vaStatus;
			vPipeTable.MinorLoss = vaLoss;

			mPipesVec.push_back(vPipeTable);

		}
		else
		{
			// 
			CivValve::ValveTable valveTable;
			valveTable.ID = vaSzNo;
			valveTable.Node1 = startNodeTable.ID;
			valveTable.Node2 = endNodeTable.ID;
			valveTable.Diameter = upperPipeDiametor;
			valveTable.Type = vaType;
			valveTable.Setting =vaSetting;
			valveTable.MinorLoss = vaLoss; 

			mValuvesVec.push_back(valveTable);
			
			// ״̬
			CivStatus::StatusTable statusTable;
			statusTable.ID = vaSzNo;
			statusTable.Setting = vaStatus;

			mStatusVec.push_back(statusTable);
		}
	}
}

void CivInpDbHelper::handleValve()
{
	handleValves2();
	return;	
}